package iterators;

import java.util.*;

public class ArrayList<T>
extends AbstractList<T>
implements RandomAccess
{
	protected T[] storage;
	protected int size;

	public ArrayList(int initSize)
	{
		if(initSize <= 0)
		{
			throw new IllegalArgumentException("Size " + initSize + " is downright silly");
		}
		storage = (T[]) new Object[initSize];
		size = 0;
	}

	public ArrayList()
	{
		this(10);
	}

	public ListIterator<T> listIterator(int index)
	{
		return new ArrayListIterator<T>(this, index);
	}

	class ArrayListIterator<T>
	implements ListIterator<T>
	{
		protected int cursore;
		protected ArrayList<T> l;
		protected int lastOp;

		ArrayListIterator(ArrayList<T> al, int index)
		{
			l = al;
			if(index < 0 || index > l.size)
			{
				throw new IndexOutOfBoundsException(String.valueOf(index));
			}
			lastOp = 0;
			cursore = index;
		}

		ArrayListIterator(ArrayList<T> al)
		{
			this(al, 0);
		}

		private ArrayListIterator()
		{
			System.err.println("Non deve mai essere chiamato il costruttore di default della classe ArrayListIterator");
			throw new RuntimeException("Non deve mai essere chiamato il costruttore di default della classe ArrayListIterator");
		}
    /**
     * Returns <tt>true</tt> if this list iterator has more elements when
     * traversing the list in the forward direction. (In other words, returns
     * <tt>true</tt> if <tt>next</tt> would return an element rather than
     * throwing an exception.)
     *
     * @return <tt>true</tt> if the list iterator has more elements when
     *		traversing the list in the forward direction.
     */
    public boolean hasNext()
		{
			return (cursore < l.size);
		}

    /**
     * Returns the next element in the list.  This method may be called
     * repeatedly to iterate through the list, or intermixed with calls to
     * <tt>previous</tt> to go back and forth.  (Note that alternating calls
     * to <tt>next</tt> and <tt>previous</tt> will return the same element
     * repeatedly.)
     *
     * @return the next element in the list.
     * @exception NoSuchElementException if the iteration has no next element.
     */
    public T next()
		{
			if(!this.hasNext())
			{
				throw new NoSuchElementException("index " + cursore + " out of bounds");
			}
			lastOp = 1;
			return l.storage[cursore++];
		}

    /**
     * Returns <tt>true</tt> if this list iterator has more elements when
     * traversing the list in the reverse direction.  (In other words, returns
     * <tt>true</tt> if <tt>previous</tt> would return an element rather than
     * throwing an exception.)
     *
     * @return <tt>true</tt> if the list iterator has more elements when
     *	       traversing the list in the reverse direction.
     */
    public boolean hasPrevious()
		{
			return (cursore > 0);
		}

    /**
     * Returns the previous element in the list.  This method may be called
     * repeatedly to iterate through the list backwards, or intermixed with
     * calls to <tt>next</tt> to go back and forth.  (Note that alternating
     * calls to <tt>next</tt> and <tt>previous</tt> will return the same
     * element repeatedly.)
     *
     * @return the previous element in the list.
     * 
     * @exception NoSuchElementException if the iteration has no previous
     *            element.
     */
    public T previous()
		{
			if(!this.hasPrevious())
			{
				throw new NoSuchElementException("index " + (cursore-1) + " out of bounds");
			}
			lastOp = -1;
			return l.storage[--cursore];
		}

    /**
     * Returns the index of the element that would be returned by a subsequent
     * call to <tt>next</tt>. (Returns list size if the list iterator is at the
     * end of the list.)
     *
     * @return the index of the element that would be returned by a subsequent
     * 	       call to <tt>next</tt>, or list size if list iterator is at end
     *	       of list. 
     */
    public int nextIndex()
		{
			return cursore;
		}

    /**
     * Returns the index of the element that would be returned by a subsequent
     * call to <tt>previous</tt>. (Returns -1 if the list iterator is at the
     * beginning of the list.)
     *
     * @return the index of the element that would be returned by a subsequent
     * 	       call to <tt>previous</tt>, or -1 if list iterator is at
     *	       beginning of list.
     */ 
    public int previousIndex()
		{
			return cursore-1;
		}


    // Modification Operations
    
    /**
     * Removes from the list the last element that was returned by
     * <tt>next</tt> or <tt>previous</tt> (optional operation).  This call can
     * only be made once per call to <tt>next</tt> or <tt>previous</tt>.  It
     * can be made only if <tt>ListIterator.add</tt> has not been called after
     * the last call to <tt>next</tt> or <tt>previous</tt>.
     *
     * @exception UnsupportedOperationException if the <tt>remove</tt>
     * 		  operation is not supported by this list iterator.
     * @exception IllegalStateException neither <tt>next</tt> nor
     *		  <tt>previous</tt> have been called, or <tt>remove</tt> or
     *		  <tt>add</tt> have been called after the last call to *
     *		  <tt>next</tt> or <tt>previous</tt>.
     */
    public void remove()
		{
			if(lastOp == 0)
			{
				throw new IllegalStateException();
			}
			if(lastOp == 1)// e' una next
			{
				int i;
				for(i=cursore;i<l.size-1;i++)
				{
					l.storage[i-1] = l.storage[i];
				}
				l.storage[i] = null;
				cursore--;
			}
			else if(lastOp == -1)//e' una previous
			{
				int i;
				for(i=cursore+1;i<l.size-1;i++)
				{
					l.storage[i-1] = l.storage[i];
				}
				l.storage[i] = null;
			}
			else// non e' 1, non e' -1 e non e' neanche 0: sono VERAMENTE nei guai
			{
				throw new IllegalStateException("Stato applicativamente inconsistente: " + lastOp);
			}
			l.size--;
			lastOp=0;
		}

    /**
     * Replaces the last element returned by <tt>next</tt> or
     * <tt>previous</tt> with the specified element (optional operation).
     * This call can be made only if neither <tt>ListIterator.remove</tt> nor
     * <tt>ListIterator.add</tt> have been called after the last call to
     * <tt>next</tt> or <tt>previous</tt>.
     *
     * @param o the element with which to replace the last element returned by
     *          <tt>next</tt> or <tt>previous</tt>.
     * @exception UnsupportedOperationException if the <tt>set</tt> operation
     * 		  is not supported by this list iterator.
     * @exception ClassCastException if the class of the specified element
     * 		  prevents it from being added to this list.
     * @exception IllegalArgumentException if some aspect of the specified
     *		  element prevents it from being added to this list.
     * @exception IllegalStateException if neither <tt>next</tt> nor
     *	          <tt>previous</tt> have been called, or <tt>remove</tt> or
     *		  <tt>add</tt> have been called after the last call to
     * 		  <tt>next</tt> or <tt>previous</tt>.
     */
    public void set(T o)
		{
			if(lastOp == 0)
			{
				throw new IllegalStateException();
			}
			if(lastOp == 1)// e' una next
			{
				l.storage[cursore-1] = o;
			}
			else if(lastOp == -1)//e' una previous
			{
				l.storage[cursore] = o;
			}
			else// non e' 1, non e' -1 e non e' neanche 0: sono VERAMENTE nei guai
			{
				throw new IllegalStateException("Stato applicativamente inconsistente: " + lastOp);
			}
			lastOp=0;
		}

    /**
     * Inserts the specified element into the list (optional operation).  The
     * element is inserted immediately before the next element that would be
     * returned by <tt>next</tt>, if any, and after the next element that
     * would be returned by <tt>previous</tt>, if any.  (If the list contains
     * no elements, the new element becomes the sole element on the list.)
     * The new element is inserted before the implicit cursor: a subsequent
     * call to <tt>next</tt> would be unaffected, and a subsequent call to
     * <tt>previous</tt> would return the new element.  (This call increases
     * by one the value that would be returned by a call to <tt>nextIndex</tt>
     * or <tt>previousIndex</tt>.)
     *
     * @param o the element to insert.
     * @exception UnsupportedOperationException if the <tt>add</tt> method is
     * 		  not supported by this list iterator.
     * 
     * @exception ClassCastException if the class of the specified element
     * 		  prevents it from being added to this list.
     * 
     * @exception IllegalArgumentException if some aspect of this element
     *            prevents it from being added to this list.
     */
    public void add(T o)
		{
			if(l.size == l.storage.length)//l'array storage e' piena
			{
				System.out.println("Da " + l.storage.length + " a " + l.storage.length*2);
				T[] tmp = (T[])(new Object[l.storage.length*2]);
				int i;
				for(i=0;i<l.storage.length;i++)
				{
					tmp[i] = l.storage[i];
				}
				l.storage = tmp;
			}

			int i;
			for(i=l.size;i>cursore;i++)
			{
				l.storage[i] = l.storage[i-1];
			}
			l.storage[i] = o;

			l.size++;
			cursore++;
			lastOp = 0;
		}
	}
}
