package iterators;

import java.util.ListIterator;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.ConcurrentModificationException;

public class LinkedList<T>
extends AbstractList<T>
{
	Node<T> head, tail;
	int size;

	Iterator<T> lastMod;
	long lastModTime;

	public LinkedList()
	{
		head = tail = null;
		size = 0;

		lastMod = null;
		lastModTime = -1;
	}

	public ListIterator<T> listIterator(int index)
	{
		return new LinkedListIterator(this, index);
	}

	public ListIterator<T> listIterator()
	{
		return this.listIterator(0);
	}

	public Iterator<T> iterator()
	{
		return this.listIterator(0);
	}

	class LinkedListIterator<T>
	implements ListIterator<T>
	{
		LinkedList<T> l;
		int pos, lastOp;
		Node<T> curr;
		long dob;

		LinkedListIterator(LinkedList theL, int index)
		{
			dob = System.currentTimeMillis();

			l = theL;
			pos = lastOp = 0;
			curr = l.head;

			for(int i = 0; i < index; i++)
			{
				next();
			}
		}

    /**
     * Returns <tt>true</tt> if this list iterator has more elements when
     * traversing the list in the forward direction. (In other words, returns
     * <tt>true</tt> if <tt>next</tt> would return an element rather than
     * throwing an exception.)
     *
     * @return <tt>true</tt> if the list iterator has more elements when
     *		traversing the list in the forward direction.
     */
    public boolean hasNext()
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}
			return curr != null;
		}

    /**
     * Returns the next element in the list.  This method may be called
     * repeatedly to iterate through the list, or intermixed with calls to
     * <tt>previous</tt> to go back and forth.  (Note that alternating calls
     * to <tt>next</tt> and <tt>previous</tt> will return the same element
     * repeatedly.)
     *
     * @return the next element in the list.
     * @exception NoSuchElementException if the iteration has no next element.
     */
    public T next()
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}
			/* Dipende dalla semantica di curr: se cambia quella devo cambiare
			 * anche questo.
			 */
			if(curr == null)
			{
				throw new NoSuchElementException("No element in position " + String.valueOf(pos));
			}
			T retval = curr.getCont();
			curr = curr.getNext();
			pos++;
			lastOp = 1;
			return retval;
		}

    /**
     * Returns <tt>true</tt> if this list iterator has more elements when
     * traversing the list in the reverse direction.  (In other words, returns
     * <tt>true</tt> if <tt>previous</tt> would return an element rather than
     * throwing an exception.)
     *
     * @return <tt>true</tt> if the list iterator has more elements when
     *	       traversing the list in the reverse direction.
     */
    public boolean hasPrevious()
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}
			//return (curr.getPrev() != null);
			// non e' corretto: se sono in fondo curr e' null anche se esiste un
			// previous
			return pos != 0;
		}

    /**
     * Returns the previous element in the list.  This method may be called
     * repeatedly to iterate through the list backwards, or intermixed with
     * calls to <tt>next</tt> to go back and forth.  (Note that alternating
     * calls to <tt>next</tt> and <tt>previous</tt> will return the same
     * element repeatedly.)
     *
     * @return the previous element in the list.
     * 
     * @exception NoSuchElementException if the iteration has no previous
     *            element.
     */
    public T previous()
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}
			if(!this.hasPrevious())
			{
				throw new NoSuchElementException("No element in position " +
				String.valueOf(pos-1));
			}
			if(pos == l.size())
			{
				curr = l.tail;
			}
			else
			{
				curr = curr.getPrev();
			}
			pos--;
			lastOp = -1;

			T retval = curr.getCont();
			return retval;

			// return curr.getCont();
		}

    /**
     * Returns the index of the element that would be returned by a subsequent
     * call to <tt>next</tt>. (Returns list size if the list iterator is at the
     * end of the list.)
     *
     * @return the index of the element that would be returned by a subsequent
     * 	       call to <tt>next</tt>, or list size if list iterator is at end
     *	       of list. 
     */
    public int nextIndex()
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}
			return pos;
		}

    /**
     * Returns the index of the element that would be returned by a subsequent
     * call to <tt>previous</tt>. (Returns -1 if the list iterator is at the
     * beginning of the list.)
     *
     * @return the index of the element that would be returned by a subsequent
     * 	       call to <tt>previous</tt>, or -1 if list iterator is at
     *	       beginning of list.
     */ 
    public int previousIndex()
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}
			return pos-1;
		}


    // Modification Operations
    
    /**
     * Removes from the list the last element that was returned by
     * <tt>next</tt> or <tt>previous</tt> (optional operation).  This call can
     * only be made once per call to <tt>next</tt> or <tt>previous</tt>.  It
     * can be made only if <tt>ListIterator.add</tt> has not been called after
     * the last call to <tt>next</tt> or <tt>previous</tt>.
     *
     * @exception UnsupportedOperationException if the <tt>remove</tt>
     * 		  operation is not supported by this list iterator.
     * @exception IllegalStateException neither <tt>next</tt> nor
     *		  <tt>previous</tt> have been called, or <tt>remove</tt> or
     *		  <tt>add</tt> have been called after the last call to *
     *		  <tt>next</tt> or <tt>previous</tt>.
     */
    public void remove()
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}

			if(lastOp == 1)//lastOp is next
			{
				if(pos == 1)
				{
					l.head = curr;
					if(l.size() == 1)
					{
						l.tail = null;
					}
					else
					{
						curr.setPrev(null);
					}
				}
				else if(pos == l.size())
				{
					l.tail = l.tail.getPrev();
					if(l.size() == 1)
					{
						l.head = null;
					}
					else
					{
						l.tail.setNext(null);
					}
				}
				else
				{
					curr.setPrev(curr.getPrev().getPrev());
					curr.getPrev().setNext(curr);
				}
				pos--;
			}
			else if(lastOp == -1)//lastOp is previous
			{
				if(pos == 0)
				{
					l.head = curr = curr.getNext();
					if(l.size() == 1)
					{
						l.tail = curr;
					}
					else
					{
						curr.setPrev(null);
					}
				}
				else if(pos == (l.size()-1))
				{
					curr = null;
					l.tail = l.tail.getPrev();
					if(l.size() == 1)
					{
						l.head = curr;
					}
					else
					{
						l.tail.setNext(null);
					}
				}
				else
				{
					curr = curr.getNext();
					curr.setPrev(curr.getPrev().getPrev());
					curr.getPrev().setNext(curr);
				}
			}
			else if(lastOp == 0)//lastOp is neither next nor previous
			{
				throw new IllegalStateException();
			}
			else
			{
				throw new IllegalStateException("Inconsistent internal state: lastOp == " + lastOp);
			}

			l.size--;
			lastOp = 0;

			l.lastMod = this;
			l.lastModTime = System.currentTimeMillis();
		}

    /**
     * Replaces the last element returned by <tt>next</tt> or
     * <tt>previous</tt> with the specified element (optional operation).
     * This call can be made only if neither <tt>ListIterator.remove</tt> nor
     * <tt>ListIterator.add</tt> have been called after the last call to
     * <tt>next</tt> or <tt>previous</tt>.
     *
     * @param o the element with which to replace the last element returned by
     *          <tt>next</tt> or <tt>previous</tt>.
     * @exception UnsupportedOperationException if the <tt>set</tt> operation
     * 		  is not supported by this list iterator.
     * @exception ClassCastException if the class of the specified element
     * 		  prevents it from being added to this list.
     * @exception IllegalArgumentException if some aspect of the specified
     *		  element prevents it from being added to this list.
     * @exception IllegalStateException if neither <tt>next</tt> nor
     *	          <tt>previous</tt> have been called, or <tt>remove</tt> or
     *		  <tt>add</tt> have been called after the last call to
     * 		  <tt>next</tt> or <tt>previous</tt>.
     */
    public void set(T o)
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}

			if(lastOp == 1)// lastOp is next
			{
				if(curr == null)
				{
					l.tail.setCont(o);
				}
				else
				{
					curr.getPrev().setCont(o);
				}
			}
			else if(lastOp == -1)// lastOp is previous
			{
				curr.setCont(o);
			}
			else if(lastOp == 0)
			{
				throw new IllegalStateException();
			}
			else
			{
				throw new IllegalStateException("Inconsistent internal state: lastOp == " + lastOp);
			}

			l.lastMod = this;
			l.lastModTime = System.currentTimeMillis();
		}

    /**
     * Inserts the specified element into the list (optional operation).  The
     * element is inserted immediately before the next element that would be
     * returned by <tt>next</tt>, if any, and after the next element that
     * would be returned by <tt>previous</tt>, if any.  (If the list contains
     * no elements, the new element becomes the sole element on the list.)
     * The new element is inserted before the implicit cursor: a subsequent
     * call to <tt>next</tt> would be unaffected, and a subsequent call to
     * <tt>previous</tt> would return the new element.  (This call increases
     * by one the value that would be returned by a call to <tt>nextIndex</tt>
     * or <tt>previousIndex</tt>.)
     *
     * @param o the element to insert.
     * @exception UnsupportedOperationException if the <tt>add</tt> method is
     * 		  not supported by this list iterator.
     * 
     * @exception ClassCastException if the class of the specified element
     * 		  prevents it from being added to this list.
     * 
     * @exception IllegalArgumentException if some aspect of this element
     *            prevents it from being added to this list.
     */
    public void add(T o)
		{
			if(l.lastMod != null && l.lastMod != this && l.lastModTime > dob)
			{
				throw new ConcurrentModificationException(l.lastMod + " <> " +
				this);
			}

			if(l.size() == 0)
			{
				l.tail = l.head = new Node<T>(o);
			}
			else
			{
				Node<T> tmp = new Node<T>(o);
				if(pos == 0)
				{
					tmp.setNext(l.head);
					l.head.setPrev(tmp);
					l.head = tmp;
				}
				else if(pos == l.size())
				{
					tmp.setPrev(l.tail);
					l.tail.setNext(tmp);
					l.tail = tmp;
				}
				else
				{
					tmp.setNext(curr);
					tmp.setPrev(curr.getPrev());
					tmp.getPrev().setNext(tmp);
					tmp.getNext().setPrev(tmp);
				}
			}
			pos++;
			l.size++;

			l.lastMod = this;
			l.lastModTime = System.currentTimeMillis();
		}
	}
}
